package de._7p.solcon.tools.coderu.core.common.aspects.performance;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.aspectj.lang.Signature;
import org.coderu.common.utils.CollectionUtils;
import org.coderu.core.common.annotations.Cached;
import org.coderu.core.common.aspects.common.aspectutils.Arguments;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public aspect Cache{ //NO_UCD

    private final Map<Arguments,Object> cache = new HashMap<Arguments,Object>();
    
    pointcut cachedCall():  execution(@Cached static * *.*(..));

    Object around() : cachedCall(){
        
        final Signature signature = thisJoinPoint.getStaticPart().getSignature();
        final String functionName = signature.getName();
        final Class<?> clazz = signature.getDeclaringType();
        final Collection<Object> args = Arrays.asList(thisJoinPoint.getArgs());
        final List<Object> signatureWithArgs = 
                CollectionUtils.concat(Arrays.asList(new Object[]{clazz,functionName}),args);
        final Arguments arguments = new Arguments(signatureWithArgs);
        Object result = cache.get(arguments);
        if(result == null){
            result = proceed();
            cache.put(arguments, result);
        }else{
            if(LOGGER.isDebugEnabled()){
                LOGGER.debug(String.format("hit cache: %s",arguments.getValue()));
            }
        }
        return result;
    }

    private static final Logger LOGGER = LoggerFactory.getLogger(Cache.class);
}
