package org.coderu.common.utils;

import javax.annotation.Nullable;

import org.coderu.common.utils.types.Pair;

import com.google.common.base.Function;


public final class PairUtils {
    /**
     * get transformer from a pair to second
     * @param <X> first type of a pair
     * @param <Y> second type of a pair
     * @return transformer from a pair to second
     */
    @SuppressWarnings("unchecked")
    public static <X,Y> Function<Pair<X,Y>,Y> pair2second(){
        return PairUtils.PAIR2SECOND;
    }

    public static <X,Y> Function<Pair<X,X>,Pair<Y,Y>> createPairTransformer(
        final Function<X,Y> f){
        return new Function<Pair<X,X>, Pair<Y,Y>>() {
            @Override
            public Pair<Y, Y> apply(@Nullable Pair<X, X> input) {
            
                return Pair.make( f.apply(input.getFirst()), f.apply(input.getSecond()));
            }
        };
    }

    public static <X,Y> Function<X,Pair<X,Y>> bindRight(final Y toBind){
        return new Function<X,Pair<X,Y>>(){public Pair<X,Y> apply(X o) {
            return Pair.make(o, toBind);
        }};
    }
    
    public static <X,Y> Function<X,Pair<Y,X>> bindLeft(final Y toBind){
        return new Function<X,Pair<Y,X>>(){public Pair<Y,X> apply(X o) {
            return Pair.make(toBind, o);
        }};
    }
    
    /**
     * get transformer from a pair to first
     * @param <X> first type of a pair
     * @param <Y> second type of a pair
     * @return transformer from a pair to first
     */
    @SuppressWarnings("unchecked")
    public static <X, Y> Function<Pair<X,Y>,X> pair2first(){
        return PairUtils.PAIR2FIRST;
    }

    @SuppressWarnings("rawtypes")
    private
    static final Function PAIR2FIRST = new Function(){
        public Object apply(Object o) {
            return ((Pair)o).getFirst();
        }
        @Override
        public String toString() {
            return "first";
        }
    };
    @SuppressWarnings("rawtypes")
    private
    static final Function PAIR2SECOND = new Function(){
        public Object apply(Object o) {
            return ((Pair)o).getSecond();
        }
        @Override
        public String toString() {
            return "second";
        }
    };
    

    private PairUtils() {
    }
}
