package org.coderu.common.utils.types;

import java.io.Serializable;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

/**
 * serves as base class for all business classes with a single member.
 * Takes care of equals, hashValue and toString methods.
 * 
 * @author arubinov
 *
 * @param <T>
 */
@SuppressWarnings("serial")
public class SingleValue<T> implements Serializable{
    
    private final T value;

    public SingleValue(T value) {
        this.value = value;
    }

    public T getValue() {
        return value;
    }
    
    @SuppressWarnings("unchecked")
    @Override
    public boolean equals(Object obj) {
        if (obj == null) { return false; }
        if (obj == this) { return true; }
        if (obj.getClass() != getClass()) {return false;}
        final SingleValue<T> rhs = (SingleValue<T>) obj;

        return value.equals(rhs.value);
    }
    
    @Override
    public int hashCode() {
        return value.hashCode();
    }

    @Override
    public String toString(){
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
        .append(value).toString();

    }
}
