package org.coderu.core.api;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang3.Validate;
import org.coderu.common.utils.types.SingleValue;


@SuppressWarnings("serial")
public final class Clazz extends SingleValue<String> implements Comparable<Clazz>{

    
    public Clazz(final String name) {
        super(name);
        checkValidClassName(name);
    }

    public Clazz(final Class<?> clazz) { // NO_UCD
        super(clazz.getName());
    }

    public String getName() {
        return getValue();
    }

    public int compareTo(Clazz o) {
        return getValue().compareTo(o.getValue());
    }

    private static void checkValidClassName(final String packageName){
        final Matcher matcher = CLASS_NAME.matcher(packageName);
        
        Validate.isTrue(
            matcher.matches(), 
            String.format("class name \"%s\" is not valid", packageName));
    }
    
    private static final Pattern CLASS_NAME = 
        Pattern.compile("(^[a-z_A-Z]+(\\.[a-z_A-Z][a-z0-9_A-Z]*)*\\.)?[a-zA-Z0-9$_-]+$");

}
