package org.coderu.core.factory;

import static org.coderu.common.utils.CollectionUtils.transform;

import java.util.Collection;
import java.util.Collections;

import org.coderu.core.api.Clazz;
import org.coderu.core.api.ClazzDependency;
import org.coderu.core.api.CompileUnit;
import org.coderu.core.api.DependencyOnSameLevelAllowed;
import org.coderu.core.api.Depth;
import org.coderu.core.api.Packagge;
import org.coderu.core.api.WrongDependencyExplorer;
import org.coderu.core.impl.common.types.ClassDependency;
import org.coderu.core.impl.explorer.DependencyChecker;

import com.google.common.base.Function;

public final class WrongDependencyExplorerFactory { // NO_UCD

    public static final WrongDependencyExplorer create(){ // NO_UCD
        return INSANCE;
    }
    
    private static final WrongDependencyExplorer INSANCE = new WrongDependencyExplorer() {

        public Collection<ClazzDependency> calculate(
                final Collection<CompileUnit> classpath, 
                final Collection<Packagge> components) {
                
                return calculate(classpath,components, Collections.<Packagge>emptyList(), Depth.UNLIMITED, DependencyOnSameLevelAllowed.YES);
            }

            public Collection<ClazzDependency> calculate(
                final Collection<CompileUnit> classpath, 
                final Collection<Packagge> components,
                final Collection<Packagge> excluded) {
                
                return calculate(classpath,components, Collections.<Packagge>emptyList(), Depth.UNLIMITED, DependencyOnSameLevelAllowed.YES);
                
            }

            public Collection<ClazzDependency> calculate(
                final Collection<CompileUnit> classpath, 
                final Collection<Packagge> components,
                final Depth depth) {
                
                return calculate(classpath,components, Collections.<Packagge>emptyList(), depth, DependencyOnSameLevelAllowed.YES);
            }

            public Collection<ClazzDependency> calculate(
                final Collection<CompileUnit> classpath, 
                final Collection<Packagge> components,
                final Collection<Packagge> excluded,
                final Depth depth){
                
                return calculate(classpath,components, excluded, depth, DependencyOnSameLevelAllowed.YES);
            }

            public Collection<ClazzDependency> calculate(
                    final Collection<CompileUnit> classpath, 
                    final Collection<Packagge> components,
                    final Collection<Packagge> excluded,
                    final Depth depth,
                    final DependencyOnSameLevelAllowed dependencyOnSameLevelAllowed) {
                
                final Collection<ClassDependency> notAllowedDependencies = 
                    DependencyChecker.calculateNotAllowedDependencies(classpath, components, excluded, depth, dependencyOnSameLevelAllowed);
                
                return transform( notAllowedDependencies,DEPENDENCY_2_API);
            }
    };

    private static final Function<ClassDependency, ClazzDependency> DEPENDENCY_2_API = 
            new Function<ClassDependency, ClazzDependency>() {
            
            public ClazzDependency apply(ClassDependency o) {
            
                return new ClazzDependency(
                    new Clazz(o.getFrom().getValue()),
                    new Clazz(o.getTo().getValue()));
            }
        };

}
