package org.coderu.core.impl.common.types;

import javax.annotation.Nullable;

import org.coderu.common.utils.types.AbstractPair;
import org.coderu.common.utils.types.Pair;
import org.coderu.core.api.Packagge;

import com.google.common.base.Function;
import com.google.common.base.Predicate;

@SuppressWarnings("serial")
public final class ClassDependency extends AbstractPair<ClassName,ClassName>{

    public static final Predicate<ClassDependency> THE_SAME_PACKAGE = new Predicate<ClassDependency>() {
        
        @Override
        public boolean apply(@Nullable ClassDependency input) {
        
            final Packagge from = ClassName.getPackage(input.getFrom());
            final Packagge to = ClassName.getPackage(input.getTo());
            return from.equals(to);
        }
    };

    public static final Predicate<ClassDependency> THE_SAME_CLASS = new Predicate<ClassDependency>() {
        
        @Override
        public boolean apply(@Nullable ClassDependency input) {
        
            final ClassName from = input.getFrom();
            final ClassName to = input.getTo();
            return ClassName.stripInnerClassPostfix(from).equals(ClassName.stripInnerClassPostfix(to));
        }
    };

    public static final Function<Pair<ClassName,ClassName>,ClassDependency> FROM_PAIR = 
        new Function<Pair<ClassName,ClassName>, ClassDependency>() {
            
            @Override
            public ClassDependency apply(@Nullable Pair<ClassName, ClassName> input) {
            
                return new ClassDependency(input.getFirst(), input.getSecond());
            }
        };
    
    public ClassDependency(ClassName from, ClassName to) {
        super(from, to);
    }

    public ClassName getFrom(){
        return getFirst();
    }

    public ClassName getTo(){
        return getSecond();
    }

}
