package org.coderu.core.impl.common.types;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;
import org.coderu.core.api.Packagge;

import com.google.common.base.Function;
import com.google.common.base.Predicate;


public final class PackageName implements Comparable<PackageName>{

    public static final PackageName ROOT = new PackageName(Collections.<String>emptyList());

     

    public static final Function<Packagge,PackageName> FROM_API = 
        new Function<Packagge, PackageName>() {
        
        public PackageName apply(Packagge o) {
        
            return new PackageName(split(o.getName()));
        }
    }; 

    

    public static Predicate<PackageName> isOncleOf(final PackageName packageName){
        return new Predicate<PackageName>() {
            
            public boolean apply(PackageName o) {
                return o.isOncleOf_(packageName);
            }
        };
    }

    

    public static Predicate<PackageName> isAncestorOf(final PackageName packageName){
        return new Predicate<PackageName>() {
            
            public boolean apply(PackageName o) {
                return o.isAncestorOf_(packageName);
            }
        };
    }
    
    public static Predicate<PackageName> isSuccessorOf(final PackageName packageName){ // NO_UCD
        return new Predicate<PackageName>() {
            
            public boolean apply(PackageName o) {
                return packageName.isAncestorOf_(o);
            }
        };
    }

    public static final Function<PackageName,String> TO_FULL_NAME = 
            new Function<PackageName, String>() {
        
        public String apply(PackageName o) {
        
            return o.getFullName();
        }
    };
    
    public static PackageName getCommonParent(final Collection<PackageName> names){
        Validate.isTrue(!names.isEmpty());
        final Iterator<PackageName> iterator = names.iterator();
        PackageName result = iterator.next();
        while(iterator.hasNext()){
            result = getCommonParent(result,iterator.next());
        }
        return result;
    }

    public static PackageName getCommonParent(final PackageName first, final PackageName second){
        final Collection<String> commonParentName = getCommonParentName(first,second);
        return new PackageName(commonParentName);
    }
    
    private static Collection<String> getCommonParentName(final PackageName first, final PackageName second){
        final Collection<String> commonParentName = new ArrayList<String>(first.name.size());
        final Iterator<String> fi = first.getName().iterator();
        final Iterator<String> si = second.getName().iterator();
        while(fi.hasNext() && si.hasNext()){
            final String fn = fi.next();
            final String sn = si.next();
            if(fn.equals(sn)){
                commonParentName.add(fn);
            }else{
                return commonParentName;
            }
        }
        return commonParentName;
    }

    private final List<String> name;
    private final String lastName;
    
    public PackageName(Collection<String> name) {
        this.name = new ArrayList<String>(name);
        final ArrayList<String> arr = new ArrayList<String>(name);
        lastName = arr.size() > 0 ? arr.get(arr.size()-1) : "";
    }

    public List<String> getName() {
        return name;
    }

    public String getLastName() {
        return lastName;
    }
    
    public String getFullName() {
        return StringUtils.join(getName(),'.');
    }

    public boolean isParentOf_(PackageName packagge) {
        return isAncestorOf_(packagge) && (packagge.getName().size() - name.size()) == 1; 
    }

    private boolean isOncleOf_(PackageName packagge) {
        return !(isAncestorOf_(packagge) || isSiblingOf_(packagge)) && 
            calculateParent(this).isAncestorOf_(packagge); 
    }

    private boolean isSiblingOf_(PackageName packagge) {
        return calculateParent(this).equals(calculateParent(packagge)); 
    }

    public boolean isAncestorOf_(PackageName packagge) {
        if(name.size() >= packagge.getName().size()){
            return false;
        }
        final Iterator<String> i1 = name.iterator();
        final Iterator<String> i2 = packagge.getName().iterator();
        
        for(int i = 0; i<name.size();i++){
            if(!i1.next().equals(i2.next())){
                return false;
            }
        }
        return true;
    }

    public static PackageName calculateParent(PackageName p) {
        return p.getName().size()>1 ? new PackageName(buildParentName(p.getName())) : ROOT;
    }

    public int compareTo(PackageName o) {
        return getFullName().compareTo(o.getFullName());
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder(3,5).append(getFullName()).toHashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) { return false; }
        if (obj == this) { return true; }
        if (obj.getClass() != getClass()) {return false;}
        final PackageName rhs = (PackageName) obj;

        return new EqualsBuilder().append(name, rhs.name).isEquals();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this,ToStringStyle.SIMPLE_STYLE)
        .append(getFullName())
        .toString();
    }

    private static Collection<String> buildParentName( Collection<String> source){
        Validate.isTrue(source.size()>1, "source.size()!>1");
        final Collection<String> result = new ArrayList<String>(source.size()-1);
        int i = 0;
        for(final String s: source){
            if(i<source.size()-1){
                result.add(s);
            }
            ++i;
        }
        return result;
    }

    private static List<String> split(final String packageName) {
        return Arrays.asList(StringUtils.split(packageName,'.'));
    }

}
