package org.coderu.core.impl.explorer.parser.impl;

import static com.google.common.base.Predicates.*;
import static org.coderu.common.utils.CollectionUtils.*;
import static org.coderu.common.utils.FunctionFactory.*;

import java.io.File;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.coderu.common.utils.MapUtils;
import org.coderu.common.utils.PairUtils;
import org.coderu.common.utils.types.Pair;
import org.coderu.core.api.Clazz;
import org.coderu.core.api.Packagge;
import org.coderu.core.impl.common.types.ClassDependencies;
import org.coderu.core.impl.common.types.PackageWithClasses;
import org.coderu.core.impl.common.types.PackageWithDependencies;
import org.coderu.core.impl.explorer.parser.impl.loader.JavaClassLoader;
import org.coderu.core.impl.explorer.parser.impl.loader.types.ByteCodeProvider;

import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.google.common.base.Predicates;

public final class DependencyParser {

    private final Predicate<Packagge> packageSelector;

    public DependencyParser(Predicate<Packagge> packageSelector) {
        this.packageSelector = packageSelector;
    }

    public Collection<PackageWithDependencies> analyse(Collection<File> files) {

        final JavaClassLoader classBuilder = new JavaClassLoader(
                packageSelector);
        final Collection<ByteCodeProvider> classes = classBuilder.build(files);

        final Collection<ByteCodeProvider> select = filter(classes,
                compose(packageSelector, ENTITY_TO_PACKAGE));

        final Collection<ClassDependencies> classDependencies = transform(
                select,
                createClassDependencyParser(CLASSPARSER, packageSelector));

        final Collection<PackageWithClasses> classDependencies2PackageDependencies = classDependencies2PackageDependencies(classDependencies);

        return toPackageWithDependencies(classDependencies2PackageDependencies);
    }

    private static Collection<PackageWithDependencies> toPackageWithDependencies(
            final Collection<PackageWithClasses> classDependencies2PackageDependencies) {

        final Collection<PackageWithDependencies> step1 = transform(
                classDependencies2PackageDependencies, TT1);

        return transform(step1,
                create1(classDependencies2PackageDependencies, step1));
    }

    private static Function<PackageWithDependencies, PackageWithDependencies> create1(
            final Collection<PackageWithClasses> list,
            final Collection<PackageWithDependencies> list2) {

        final Map<Packagge, PackageWithClasses> mapByPackage = MapUtils.mapBy(
                list, PackageWithClasses.TO_PACKAGE);

        return new Function<PackageWithDependencies, PackageWithDependencies>() {

            public PackageWithDependencies apply(PackageWithDependencies o) {

                final Packagge first = o.getFirst();
                final Collection<Packagge> dependsOn = mapByPackage.get(first)
                        .getThird();
                o.addDependencies(filter(
                        list2,
                        compose(Predicates.in(dependsOn),
                                PackageWithDependencies.TO_PACKAGE)));
                return o;
            }
        };
    }

    private static final Function<PackageWithClasses, PackageWithDependencies> TT1 = new Function<PackageWithClasses, PackageWithDependencies>() {

        public PackageWithDependencies apply(PackageWithClasses o) {

            final Packagge first = o.getFirst();
            final Collection<ClassDependencies> second = o.getSecond();
            return new PackageWithDependencies(first, second);
        }
    };

    private static Collection<PackageWithClasses> classDependencies2PackageDependencies(
            final Collection<ClassDependencies> classDependencies) {

        final List<Pair<Packagge, Collection<ClassDependencies>>> groupByPackage = MapUtils
                .map2keyValueList(MapUtils.groupBy(classDependencies,
                        CLASSDEPENDENCIES_TO_PACKAGGE));

        final Collection<Pair<Pair<Packagge, Collection<ClassDependencies>>, Collection<Packagge>>> result = transform(
                groupByPackage,
                remainSource(concat(PAIR_2_SECOND, CLASSDEP_TO_PACKDEP)));

        return transform(result, TO_PACKAGEWITHCLASSES);
    }

    private static final Function<Pair<Packagge, Collection<ClassDependencies>>, Collection<ClassDependencies>> PAIR_2_SECOND = PairUtils
            .pair2second();

    private final static Function<Pair<Pair<Packagge, Collection<ClassDependencies>>, Collection<Packagge>>, PackageWithClasses> TO_PACKAGEWITHCLASSES = new Function<Pair<Pair<Packagge, Collection<ClassDependencies>>, Collection<Packagge>>, PackageWithClasses>() {

        public PackageWithClasses apply(
                Pair<Pair<Packagge, Collection<ClassDependencies>>, Collection<Packagge>> o) {

            final Packagge first = o.getFirst().getFirst();
            final Collection<ClassDependencies> second = o.getFirst()
                    .getSecond();
            final Collection<Packagge> second2 = o.getSecond();
            return new PackageWithClasses(first, second, second2);
        }
    };

    private final static Function<Collection<ClassDependencies>, Collection<Packagge>> CLASSDEP_TO_PACKDEP = new Function<Collection<ClassDependencies>, Collection<Packagge>>() {

        public Collection<Packagge> apply(Collection<ClassDependencies> o) {

            final Collection<Clazz> col = concat(transform(o,
                    ClassDependencies.TO_DEPENDENCIES));

            return transform(removeDuplicates(col), CLAZZ_TO_PACKAGE);
        }
    };

    private static Function<ByteCodeProvider, ClassDependencies> createClassDependencyParser(
            final ClassParser classParser,
            final Predicate<Packagge> packageSelector) {

        return new Function<ByteCodeProvider, ClassDependencies>() {

            public ClassDependencies apply(ByteCodeProvider o) {

                final Collection<Clazz> classes = classParser.apply(o);
                final Clazz clazz = ENTITY_TO_CLASS.apply(o);
                filter(classes,
                        Predicates.compose(packageSelector, CLAZZ_TO_PACKAGE));
                return new ClassDependencies(clazz, classes);
            }
        };
    }

    private static final Function<ByteCodeProvider, Packagge> ENTITY_TO_PACKAGE = new Function<ByteCodeProvider, Packagge>() {

        public Packagge apply(ByteCodeProvider o) {

            final String name = o.getName();
            final int indexOfLastSlash = name
                    .lastIndexOf(IOUtils.DIR_SEPARATOR_UNIX);
            if (indexOfLastSlash == -1) {
                return new Packagge("");
            } else {
                final String path = name.substring(0, indexOfLastSlash);
                final Packagge packagge = new Packagge(path.replace(
                        IOUtils.DIR_SEPARATOR_UNIX, '.'));
                return packagge;
            }
        }
    };

    private static final Function<ByteCodeProvider, Clazz> ENTITY_TO_CLASS = new Function<ByteCodeProvider, Clazz>() {

        public Clazz apply(ByteCodeProvider o) {

            final String name = o.getName();
            final String withoutExtention = name.substring(0, name.length()
                    - CLASS_EXTENTION_LENGTH);
            return new Clazz(withoutExtention.replace(
                    IOUtils.DIR_SEPARATOR_UNIX, '.'));
        }
    };

    private static final int CLASS_EXTENTION_LENGTH = ".class".length();

    private static final Function<Clazz, Packagge> CLAZZ_TO_PACKAGE = new Function<Clazz, Packagge>() {

        public Packagge apply(Clazz o) {

            final String name = o.getName();
            final int indexOfLastPoint = name.lastIndexOf('.');
            return indexOfLastPoint != -1 ? new Packagge(name.substring(0,
                    indexOfLastPoint)) : new Packagge("");
        }
    };

    private static final Function<ClassDependencies, Packagge> CLASSDEPENDENCIES_TO_PACKAGGE = concat(
            ClassDependencies.TO_CLAZZ, CLAZZ_TO_PACKAGE);

    private static final ClassParser CLASSPARSER = new ClassParser();
}
